package cc.fascinated.bat.command;

import lombok.AllArgsConstructor;
import lombok.Getter;
import net.dv8tion.jda.api.entities.emoji.Emoji;

import java.util.Arrays;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor
@Getter
public enum Category {
    GENERAL(Emoji.fromUnicode("U+2699"), "General", false),
    FUN(Emoji.fromFormatted("U+1F973"), "Fun", false),
    SERVER(Emoji.fromFormatted("U+1F5A5"), "Server", false),
    MODERATION(Emoji.fromFormatted("U+1F6E0"), "Moderation", false),
    UTILITY(Emoji.fromFormatted("U+1F6E0"), "Utility", false),
    MUSIC(Emoji.fromFormatted("U+1F3B5"), "Music", false),
    MOVIES_TV(Emoji.fromFormatted("U+1F3A5"), "Movies & TV", false),
    MESSAGES(Emoji.fromFormatted("U+1F4A3"), "Messages", false),
    LOGS(Emoji.fromFormatted("U+1F4D1"), "Logs", false),
    BEAT_SABER(Emoji.fromFormatted("U+1FA84"), "Beat Saber", false),
    BOT_ADMIN(null, null, true);

    /**
     * The emoji for the category
     */
    private final Emoji emoji;

    /**
     * The name of the category
     */
    private final String name;

    /**
     * If the category is hidden
     */
    private final boolean hidden;

    /**
     * Gets a category by its name
     *
     * @param name the name of the category
     * @return the category
     */
    public static Category getByName(String name) {
        for (Category category : Category.values()) {
            if (category.getName().equalsIgnoreCase(name)) {
                return category;
            }
        }
        return null;
    }

    /**
     * Gets all the visible categories
     *
     * @return the visible categories
     */
    public static List<Category> getCategories() {
        return Arrays.stream(Category.values()).filter(category -> !category.isHidden()).toList();
    }
}
