package cc.fascinated.bat.features.drag.listeners.request;

import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.event.EventListener;
import cc.fascinated.bat.features.drag.DragRequest;
import cc.fascinated.bat.features.drag.command.RequestSubCommand;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.events.interaction.component.ButtonInteractionEvent;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
public class TargetChannelListener implements EventListener {
    @Override
    public void onButtonInteraction(BatGuild guild, @NonNull BatUser user, @NonNull ButtonInteractionEvent event) {
        User buttonUser = event.getUser();
        Member member = guild.getDiscordGuild().getMember(buttonUser);
        if (member == null) return;

        DragRequest joinRequest = RequestSubCommand.JOIN_REQUESTS.stream()
                .filter(request -> request.getVoiceChannel().getId().equals(event.getChannel().getId()))
                .findFirst()
                .orElse(null);
        if (joinRequest == null) return;

        if (event.getComponentId().equals("drag-request-accept")) {
            joinRequest.getVoiceChannel().getGuild().moveVoiceMember(joinRequest.getMember(), joinRequest.getVoiceChannel()).queue();
            joinRequest.getInteractionHook().retrieveOriginal().queue(message -> {
                message.editMessageEmbeds(EmbedUtils.successEmbed()
                        .setDescription("%s has accepted your request to join their voice channel!".formatted(joinRequest.getTarget().getAsMention()))
                        .build()).queue(then -> then.delete().queueAfter(10, java.util.concurrent.TimeUnit.SECONDS));
                message.editMessageComponents().queue();
            });
            event.replyEmbeds(EmbedUtils.successEmbed()
                            .setDescription("You have accepted %s's request to join your voice channel!".formatted(joinRequest.getMember().getAsMention()))
                            .build())
                    .queue();
        } else if (event.getComponentId().equals("drag-request-decline")) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("You have declined %s's request to join your voice channel!".formatted(joinRequest.getMember().getAsMention()))
                            .build())
                    .queue();
            joinRequest.getInteractionHook().retrieveOriginal().queue(message -> {
                message.editMessageEmbeds(EmbedUtils.errorEmbed()
                        .setDescription("%s has declined your request to join their voice channel.".formatted(joinRequest.getTarget().getAsMention()))
                        .build()).queue();
                message.editMessageComponents().queue();
            });
        }
        RequestSubCommand.JOIN_REQUESTS.remove(joinRequest);
        // Remove the buttons from the embed
        event.getInteraction().getMessage().editMessageComponents().queue();
    }
}
