package cc.fascinated.bat.features.logging;

import cc.fascinated.bat.common.EnumUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;
import net.dv8tion.jda.api.entities.emoji.Emoji;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor @Getter
public enum LogCategory {
    MESSAGE(Emoji.fromUnicode("📩")),
    MEMBER(Emoji.fromUnicode("👤")),
    CHANNEL(Emoji.fromUnicode("📺")),
    GUILD(Emoji.fromUnicode("🏰"));

    /**
     * The emoji of the log category
     */
    private final Emoji emoji;

    /**
     * Gets the name of the log category
     *
     * @return the name
     */
    public String getName() {
        return EnumUtils.getEnumName(this);
    }

    /**
     * Gets the log category by the name
     *
     * @param name - the name
     * @return the log category, or null if it doesn't exist
     */
    public static LogCategory getLogCategory(String name) {
        for (LogCategory logCategory : values()) {
            if (logCategory.getName().equalsIgnoreCase(name)) {
                return logCategory;
            }
        }
        return null;
    }
}
