package cc.fascinated.bat.features.logging;

import cc.fascinated.bat.common.EnumUtils;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.util.ArrayList;
import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@AllArgsConstructor @Getter
public enum LogType {
    /**
     * Message Events
     */
    MESSAGE_DELETE(LogCategory.MESSAGE),
    MESSAGE_EDIT(LogCategory.MESSAGE),

    /**
     * Member Events
     */
    MEMBER_JOIN(LogCategory.MEMBER),
    MEMBER_LEAVE(LogCategory.MEMBER),
    MEMBER_NICKNAME_UPDATE(LogCategory.MEMBER),
    MEMBER_GLOBAL_NAME_UPDATE(LogCategory.MEMBER),
    MEMBER_USERNAME_UPDATE(LogCategory.MEMBER),
    MEMBER_AVATAR_UPDATE(LogCategory.MEMBER),
    MEMBER_ROLE_UPDATE(LogCategory.MEMBER),
    MEMBER_BAN(LogCategory.MEMBER),
    MEMBER_UNBAN(LogCategory.MEMBER),
    MEMBER_TIMEOUT(LogCategory.MEMBER),
    MEMBER_BOOSTED(LogCategory.MEMBER),
    MEMBER_BOOST_EXPIRE(LogCategory.MEMBER),

    /**
     * Channel Events
     */
    CHANNEL_CREATE(LogCategory.CHANNEL),
    CHANNEL_DELETE(LogCategory.CHANNEL),
    VOICE_CHANNEL_JOIN(LogCategory.CHANNEL),
    VOICE_CHANNEL_LEAVE(LogCategory.CHANNEL),
    CHANNEL_UPDATE_NAME(LogCategory.CHANNEL),

    /**
     * Guild Events
     */
    INVITE_CREATE(LogCategory.GUILD),
    EMOJI_ADD(LogCategory.GUILD),
    EMOJI_REMOVE(LogCategory.GUILD),
    EMOJI_RENAME(LogCategory.GUILD);

    /**
     * The category of the log type
     */
    private final LogCategory category;

    /**
     * The name of the log type
     */
    public String getName() {
        return EnumUtils.getEnumName(this);
    }

    /**
     * Gets the log type by the name
     *
     * @param name - the name
     * @return the log type, or null if it doesn't exist
     */
    public static LogType getLogType(String name) {
        for (LogType logType : values()) {
            if (logType.getName().equalsIgnoreCase(name)) {
                return logType;
            }
        }
        return null;
    }

    /**
     * Gets the log types by the category
     *
     * @param category - the category
     * @return the log types
     */
    public static List<LogType> getLogTypesByCategory(String category) {
        List<LogType> logTypes = new ArrayList<>();
        for (LogType logType : values()) {
            if (logType.getCategory().getName().equalsIgnoreCase(category)) {
                logTypes.add(logType);
            }
        }
        return logTypes;
    }
}
