package cc.fascinated.bat.features.logging.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.logging.LogCategory;
import cc.fascinated.bat.features.logging.LogProfile;
import cc.fascinated.bat.features.logging.LogType;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Fascinated (fascinated7)
 */
@Component("logs:remove.sub")
@CommandInfo(name = "remove", description = "Remove the channel for a log type")
public class RemoveSubCommand extends BatSubCommand {
    public RemoveSubCommand() {
        super.addOption(OptionType.STRING, "type", "The type of log to remove", true);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        OptionMapping typeOption = event.getOption("type");
        if (typeOption == null) {
            return;
        }
        String type = typeOption.getAsString();
        LogProfile profile = guild.getLogProfile();

        // Remove the log channel for all log types
        if (type.equalsIgnoreCase("all")) {
            for (LogType logType : LogType.values()) {
                profile.removeLogChannel(logType);
            }

            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription("%s Successfully removed the log channel for all log types".formatted(Emojis.CHECK_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        // Remove the log channel for a specific log category
        LogCategory logCategory = LogCategory.getLogCategory(type);
        if (logCategory != null) {
            List<LogType> category = LogType.getLogTypesByCategory(type);
            EmbedDescriptionBuilder description = new EmbedDescriptionBuilder(null);
            description.appendLine("%s Successfully removed the log channel for the `%s` category".formatted(
                    Emojis.CHECK_MARK_EMOJI,
                    logCategory.getName()
            ), false);
            description.emptyLine();

            int removed = 0;
            for (LogType logType : category) {
                if (!profile.hasLogChannel(logType)) {
                    continue;
                }
                description.appendLine(logType.getName(), true);
                profile.removeLogChannel(logType);
                removed++;
            }

            event.replyEmbeds(EmbedUtils.successEmbed()
                    .setDescription(removed == 0 ? "No log channels were removed for the `%s` category".formatted(logCategory.getName()) : description.build())
                    .build()).queue();
            return;
        }

        // Remove the log channel for a specific log type
        LogType logType = LogType.getLogType(type);
        if (logType == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("%s Invalid log type".formatted(Emojis.CROSS_MARK_EMOJI))
                    .build()).queue();
            return;
        }

        if (!profile.hasLogChannel(logType)) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("The log channel for `%s` is not set".formatted(logType.getName()))
                    .build()).queue();
            return;
        }
        profile.removeLogChannel(logType);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("%s Successfully removed the log channel for `%s`".formatted(
                        Emojis.CHECK_MARK_EMOJI,
                        logType.getName()
                )).build()).queue();
    }
}
