package cc.fascinated.bat.features.messagesnipe.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.common.PasteUtils;
import cc.fascinated.bat.features.messagesnipe.MessageSnipeFeature;
import cc.fascinated.bat.features.messagesnipe.SnipedMessage;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.User;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@CommandInfo(name = "deleted", description = "Snipe the last deleted message in this channel")
public class DeletedSubCommand extends BatSubCommand {
    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        SnipedMessage message = MessageSnipeFeature.getDeletedMessage(guild, channel.getId());
        if (message == null) {
            event.replyEmbeds(EmbedUtils.errorEmbed()
                    .setDescription("There are no deleted messages to snipe in this channel")
                    .build()).queue();
            return;
        }

        User author = message.getMessage().getAuthor();
        String content = message.getMessage().getContentStripped();
        String formattedContent = content.length() > 512 ? PasteUtils.uploadPaste(content).getUrl() : "```\n%s\n```".formatted(content);
        event.replyEmbeds(EmbedUtils.genericEmbed()
                .setDescription(new EmbedDescriptionBuilder("Deleted Message Snipe")
                        .appendLine("Author: **%s** (%s)".formatted(author.getAsMention(), author.getId()), true)
                        .appendLine("Deleted: <t:%d:R>".formatted(message.getDeletedDate().getTime() / 1000), true)
                        .appendLine("Content: %s".formatted(formattedContent), true)
                        .build()).build()).queue();
    }
}
