package cc.fascinated.bat.features.welcomer.command;

import cc.fascinated.bat.Emojis;
import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedDescriptionBuilder;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.welcomer.WelcomerPlaceholders;
import cc.fascinated.bat.features.welcomer.WelcomerProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Fascinated (fascinated7)
 */
@Component("welcomer:message.sub")
@CommandInfo(name = "message", description = "Set the welcomer message (this will remove the welcomer embed if set)")
public class MessageSubCommand extends BatSubCommand {
    public MessageSubCommand() {
        super.addOption(OptionType.STRING, "message", "The message to send", true);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction event) {
        WelcomerProfile profile = guild.getWelcomerProfile();
        OptionMapping messageOption = event.getOption("message");
        if (messageOption == null) {
            return;
        }
        String message = messageOption.getAsString();
        boolean isEmbedEnabled = profile.isEmbed();
        profile.setMessage(message);

        EmbedDescriptionBuilder description = new EmbedDescriptionBuilder("Welcomer Message")
                .appendLine("%s Set the message to `%s`".formatted(Emojis.CHECK_MARK_EMOJI, message), false);
        if (isEmbedEnabled) {
            description.appendLine("*This has removed the embed welcomer*", false);
        }
        description.emptyLine();
        description.appendLine("**Preview:** %s".formatted(WelcomerPlaceholders.replaceAllPlaceholders(message, guild, user)), false);
        event.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription(description.build())
                .build()).queue();
    }
}
